# Mobile Verification Toolkit (MVT)
# Copyright (c) 2021-2023 The MVT Authors.
# Use of this software is governed by the MVT License 1.1 that can be found at
#   https://license.mvt.re/1.1/

from mvt.common.module import MVTModule

from .aqf_files import AQFFiles
from .aqf_getprop import AQFGetProp
from .aqf_log_timestamps import AQFLogTimestamps
from .aqf_packages import AQFPackages
from .aqf_processes import AQFProcesses
from .aqf_settings import AQFSettings
from .mounts import Mounts
from .root_binaries import RootBinaries

ANDROIDQF_MODULES: list[type[MVTModule]] = [
    AQFPackages,
    AQFProcesses,
    AQFGetProp,
    AQFSettings,
    AQFFiles,
    AQFLogTimestamps,
    RootBinaries,
    Mounts,
]
